/*
================================================================================
 File: main.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       
 Compiler:      GCC
 Hardware:      STM32
 Doc:           
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:      
================================================================================
 */

#include <stdint.h>
#include "stm32f10x_conf.h"

// application components
#include "converter.h"
#include "usbd.h"


#define LED1_PORT						GPIOC
#define LED1_PIN						GPIO_Pin_12
#define LED1_PIN_MODE					GPIO_Mode_Out_PP


static USBDC_params USBDC;


void RCC_Configuration(void)
{
  /* Setup the microcontroller system. Initialize the Embedded Flash Interface,  
     initialize the PLL and update the SystemFrequency variable. */
	SystemInit();
   
  /* Enable GPIO Ports and AFIO clock */
	RCC_APB2PeriphClockCmd(
			RCC_APB2Periph_GPIOA
			| RCC_APB2Periph_GPIOB
			| RCC_APB2Periph_GPIOC
			| RCC_APB2Periph_GPIOD
			| RCC_APB2Periph_AFIO
			, ENABLE);
}

/**
  * @brief  Configures the different GPIO ports.
  * @param  None
  * @retval : None
  */
void GPIO_Configuration(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;

	// LED initialize
	GPIO_InitStructure.GPIO_Pin = LED1_PIN;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = LED1_PIN_MODE;
	GPIO_Init(LED1_PORT, &GPIO_InitStructure);
}

/**
  * @brief  Configures the different IO ports.
  * @param  None
  * @retval : None
  */
void IO_Configuration(void)
{
	
}

/**
  * @brief  Configure the nested vectored interrupt controller.
  * @param  None
  * @retval : None
  */
void NVIC_Configuration(void)
{
//	NVIC_InitTypeDef NVIC_InitStructure;
  
	/* Configure one bit for preemption priority */
//	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	NVIC_PriorityGroupConfig( NVIC_PriorityGroup_4 );
  
//	NVIC_InitStructure.NVIC_IRQChannel = EXTI0_IRQn;
//	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
//	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
//	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
//	NVIC_Init(&NVIC_InitStructure);
}


void wait(void) {
	int i;
	for(i=0; i<1000000; i++) {
		asm("nop");
	}
}



void system_initialize(void) {
	ErrorStatus HSEStartUpStatus;
	uint32_t i;

	/* RCC system reset(for debug purpose) */
	RCC_DeInit();

	/* Enable HSE */
	RCC_HSEConfig(RCC_HSE_ON);

	//default HSEStartUp_TimeOut is too short for out 4MHz oscillator to start (~80ms)
	for (i = 0; i < 0xFFFF; i++){
		asm("nop");
	}
	
	/* Wait till HSE is ready */
	HSEStartUpStatus = RCC_WaitForHSEStartUp();

	if(HSEStartUpStatus == SUCCESS)	{
		/* Enable Prefetch Buffer */
		FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);

		/* Flash 2 wait state */
		FLASH_SetLatency(FLASH_Latency_2);

		/* HCLK = SYSCLK */
		RCC_HCLKConfig(RCC_SYSCLK_Div1); 

		/* PCLK2 = HCLK */
		RCC_PCLK2Config(RCC_HCLK_Div4); 

		/* PCLK1 = HCLK/2 */
		RCC_PCLK1Config(RCC_HCLK_Div2);	// SPI2: max div = 4 /4MHz HSE

		/* PLLCLK = 8MHz * 6 = 48 MHz */
		RCC_PLLConfig(RCC_PLLSource_HSE_Div2, RCC_PLLMul_12);

		/* Enable PLL */ 
		RCC_PLLCmd(ENABLE);

		/* Wait till PLL is ready */
		while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET) {
		}

		/* Select PLL as system clock source */
		RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);

		/* Wait till PLL is used as system clock source */
		while(RCC_GetSYSCLKSource() != 0x08) {
		}
	}

#ifdef  VECT_TAB_RAM  
	/* Set the Vector Table base location at 0x20000000 */ 
	NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0); 
#else  /* VECT_TAB_FLASH  */
	/* Set the Vector Table base location at 0x08000000 */ 
	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);   
	NVIC_PriorityGroupConfig( NVIC_PriorityGroup_4 );
	
#endif
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD | RCC_APB2Periph_AFIO, ENABLE);
}


static void main_event(USBDC_params *USBDC, uint8_t flags, USBDE_params *event) {
	if(flags == USBDEF_SUSPENDED) {
//		GPIO_SetBits(LED1_PORT, LED1_PIN);
	}
	if(flags == USBDEF_WAKEUP) {
//		GPIO_ResetBits(LED1_PORT, LED1_PIN);
	}
}


// -----------------------------------------------------------------------------
//  main
// -----------------------------------------------------------------------------
int main(void)
{
	// initialize clocks, interrupts and other things
//	system_initialize();
	
	// System Clocks Configuration
	RCC_Configuration();

	// NVIC configuration
	NVIC_Configuration();

	// Configure the GPIO ports
	GPIO_Configuration();
	GPIO_SetBits(LED1_PORT, LED1_PIN);

	// Configure the IO ports
	IO_Configuration();
	
	USBDC_init(&USBDC);
	USBDE_install(main_event, USBDEM_SUSPENDED | USBDEM_WAKEUP, &USBDC);
	USBDZC_install(&USBDC);

	// usbd to rs232c converter initialize
	converter_init();

	while(1) {
		GPIO_SetBits(LED1_PORT, LED1_PIN);
		wait();
		GPIO_ResetBits(LED1_PORT, LED1_PIN);
		wait();
	}

} /* main */

/*! \file main.c
    \brief Main application's entry point

*/

