/*
================================================================================
 File: cdc_vcom.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USB virtual serial port based on USB CDC device class
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#include <string.h>

#ifndef CDC_VCOM_H_
#include "cdc_vcom.h"
#endif

/** \defgroup vCOM virtual COM library

@{*/



// -----------------------------------------------------------------------------
//  CDC_Control_close_transmition
// -----------------------------------------------------------------------------
static void CDC_control_close_transmition (USBDC_params *usbdc, USBDEP_params *ep) {
	if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
		USBD_send_status(0, 0, usbdc);
	}
	else {
		ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
	}
	usbdc->ep_irq[0] = 0;
} /* CDC_control_close_transmition */

// -----------------------------------------------------------------------------
//  CDC_SET_line_coding_close_transmition
// -----------------------------------------------------------------------------
static void CDC_set_line_coding_close_transmition (USBDC_params *usbdc, USBDEP_params *ep) {
	CDC_VCOM_params *vcom;
	vcom = (CDC_VCOM_params *)(usbdc->interfaces[usbdc->upper_layer_data.uint8].upper_layer_data.pvoid);
	
	if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
		USBD_send_status(0, 0, usbdc);
	}
	else {
		ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
	}

	usbdc->ep_irq[0] = 0;

	if(vcom->change_params_irq) {
		(*vcom->change_params_irq) (vcom);
	}
} /* CDC_set_line_coding_close_transmition */






// -----------------------------------------------------------------------------
//  CCI_Control_handler
// -----------------------------------------------------------------------------
static void CDC_CCI_control_handler (USBDC_params *usbdc, USBDEP_params *ep, uint8_t interface, USBDR_params *request) {
	CDC_VCOM_params *vcom = (CDC_VCOM_params *)(usbdc->interfaces[interface].upper_layer_data.pvoid);
	uint8_t send_order = 0;
	uint16_t USBD_COM_reserved_buffer;

	usbdc->upper_layer_data.uint8 = interface;

	// check which request we received
	switch(*((uint16_t*)&(request->Request.WholeRequestValue))) {
		// this two requests are not supported
		// because are never used
//		case CDC_SEND_ENCAPSULATED_COMMAND_INTERFACE :
//			break;

//		case CDC_GET_ENCAPSULATED_RESPONSE_INTERFACE :
//			break;


	
		case CDC_SET_COMM_FEATURE_INTERFACE :
			switch(request->wValue) {
				// abstract_state
				case 0x01 :
					// receive abstract_state
					USBD_recv_data_out ((uint8_t*)&(vcom->abstract_state), 2, 0, usbdc);
					// if receive process ended
					if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)){
						if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
							USBD_send_status(0, 0, usbdc);
						}
						else {
							ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
						}
					}
					// receive is processing
					// CDC_control_close_transmition function will close
					// control transfer as result of DATA OUT end_of_transfer
					// event handler
					else {
						usbdc->ep_irq[0] = CDC_control_close_transmition;
					}
					break;

				// country_setting
				case 0x02 :
					USBD_recv_data_out ((uint8_t*)&(vcom->country_setting), 2, 0, usbdc);
					// if receive process ended
					if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)){
						if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
							USBD_send_status(0, 0, usbdc);
						}
						else {
							ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
						}
					}
					// receive is processing
					// CDC_control_close_transmition function will close
					// control transfer as result of DATA OUT end_of_transfer
					// event handler
					else {
						usbdc->ep_irq[0] = CDC_control_close_transmition;
					}
					break;

				default :
					break;
			}
			break;




		case CDC_GET_COMM_FEATURE_INTERFACE :
			// check which FEATURE Host wanted to receive
			switch(request->wValue) {
				// abstract_state
				case 0x01 :
					USBD_send_data_in ((uint8_t*)&(vcom->abstract_state), 2, 0, usbdc);
					break;

				// country_setting
				case 0x02 :
					USBD_send_data_in ((uint8_t*)&(vcom->country_setting), 2, 0, usbdc);
					break;

				default :
					break;
			}
			break;




		case CDC_CLEAR_COMM_FEATURE_INTERFACE :
			// check which FEATURE Host wanted to clear
			switch(request->wValue) {
				// abstract_state
				case 0x01 :
					vcom->abstract_state = 0;
					USBD_send_status(0, 0, usbdc);
					break;

				// country_setting
				case 0x02 :
					vcom->country_setting = 0;
					USBD_send_status(0, 0, usbdc);
					break;

				default :
					break;
			}
			break;




		case CDC_SET_LINE_CODING_INTERFACE :
			// receive line_coding
			USBD_recv_data_out ((uint8_t*)&(vcom->line_coding), request->wLength, 0, usbdc);
			// if receive process ended
			if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)){
				if(ep->transfer_params.transfer_state != USBD_TOO_LESS_SPACE) {
					USBD_send_status(0, 0, usbdc);
	
					if(vcom->change_params_irq) {
						(*vcom->change_params_irq) (vcom);
					}
				}
				else {
					ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
				}
			}
			// receive is processing
			// CDC_set_line_coding_close_transmition function will close
			// control transfer as result of DATA OUT end_of_transfer
			// event handler
			else {
				usbdc->ep_irq[0] = CDC_set_line_coding_close_transmition;
			}
			break;




		case CDC_GET_LINE_CODING_INTERFACE :
			USBD_send_data_in ((uint8_t*)&(vcom->line_coding), request->wLength, 0, usbdc);
			break;




		case CDC_SET_CONTROL_LINE_STATE_INTERFACE :
			// at first we have to keep current control_line_state
			// because we will compare new control_line_state with this one
			USBD_COM_reserved_buffer = vcom->control_line_state;
			// update control_line_state
			vcom->control_line_state = request->wValue;
			USBD_send_status(0, 0, usbdc);

			// if line DTR has been changed
			if((USBD_COM_reserved_buffer & 0x1) != (vcom->control_line_state & 0x1)) {
				// if we declared DTR_handler
				if(vcom->dtr_irq) {
					// then we call this function
					(*vcom->dtr_irq) (vcom);
				}
#if(CDC_DTR_AUTO_REPEAT_SUPPORT == USBD_FEATURE_ON)
				// DTR_handler was not declared - auto repeat
				else {
					send_order = 1;
					if(CDC_VCOM_get_DTR(vcom)) {
						CDC_VCOM_set_DSR(vcom);
					}
					else {
						CDC_VCOM_clr_DSR(vcom);
					}
				}
#endif
			}
			// if line RTS has been changed
			if((USBD_COM_reserved_buffer & 0x2) != (vcom->control_line_state & 0x2)) {
				// if we declared RTS_handler
				if(vcom->rts_irq) {
					// then we call this function
					(*vcom->rts_irq) (vcom);
				}
#if(CDC_RTS_AUTO_REPEAT_SUPPORT == USBD_FEATURE_ON)
				// RTS_handler was not declared - auto repeat
				else {
					send_order = 1;
					if(CDC_VCOM_get_RTS(vcom)) {
						CDC_VCOM_set_DCD(vcom);
					}
					else {
						CDC_VCOM_clr_DCD(vcom);
					}
				}
#endif
			}
#if((CDC_DTR_AUTO_REPEAT_SUPPORT == USBD_FEATURE_ON) || (CDC_RTS_AUTO_REPEAT_SUPPORT == USBD_FEATURE_ON))
			// if we have NOTIFICATION to send
			if(send_order) {
				// send NOTIFICATION
				USBD_send_data_in(
						(uint8_t*)&(vcom->serial_state_notification),
						10,
						vcom->ep_notif_number,
						usbdc);
			}
#endif

			break;




		case CDC_SEND_BREAK_INTERFACE :
			vcom->duration_of_break = request->wValue;
			USBD_send_status(0, 0, usbdc);
			break;




		default:
			USBD_send_stall(0, usbdc);
			break;
	}
} /* CDC_CCI_control_handler */





static void CDC_event_handler(USBDC_params *usbdc, uint8_t flags, USBDE_params *event) {
	USBDEP_params *ep;
	USBDI_params *interface;
	uint8_t counter;
	uint8_t EP_number;
	union {
		void *pvoid;
		void (*hvoid)(USBDC_params *, uint8_t, USBDE_params *);
	}owner;
	if(flags == USBDEF_SOF) {
		for(counter = 0; counter < USBD_MAX_NUM_INTERFACES; counter++) {
			interface = &(usbdc->interfaces[counter]);
			owner.pvoid = interface->owner;
			if(owner.hvoid == CDC_event_handler) {
				EP_number = ((CDC_VCOM_params *)(interface->upper_layer_data.pvoid))->ep_in_number;
				ep = USBDEP_get_params(EP_number, usbdc);
				if(!(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
					if(usbdc->ep_irq[EP_number]) {
						(*usbdc->ep_irq[EP_number]) (usbdc, ep);
					}
				}
			}
		}
	}
}




#if(CDC_OUT_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
//	-----------------------------------------------------------------------------
/**
 *  CDC_out_handler
 *  Interrupt handler 
 *  USB layer generate this function when endpoint receives some data from Host
 */
//	-----------------------------------------------------------------------------
static void CDC_out_handler (USBDC_params *usbdc, USBDEP_params *ep) {
	CDC_VCOM_params *vcom = usbdc->interfaces[ep->interface].upper_layer_data.pvoid;

	if(!vcom->out_irq || !vcom->out_buffer) {
		USBDEP_data_out_ready(ep->number, usbdc);
		ep->transfer_params.transfer_state = USBD_CLEAR_ALL_FLAGS;
		return;
	}
	// at first we have to check the state of Endpoint - if USBD_Idle_or_background
	// state was set by USBD and USBD_TRANSFER_PRESENT bit is reset
	// then we know that there is some piece of data in Endpoint
	// and we were not declared the space for receiving process
	if((ep->transfer_params.transfer_state == USBD_IDLE_OR_BACKGROUND) && !(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT)) {
		// reload data to vcom->out_buffer
		USBD_recv_data_out(
				vcom->out_buffer,
				ep->transfer_params.data_buffered_length,
				vcom->ep_out_number,
				usbdc);
	}

	// if we ended reload process we can call user handler
	if((ep->transfer_params.transfer_state == USBD_END_OF_TRANSFER)
			|| (ep->transfer_params.transfer_state == USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET)) {

		if(!vcom->out_irq(vcom, vcom->out_buffer, ep->transfer_params.data_transferred_length)) {
			USBDEP_data_out_ready(ep->number, usbdc);
		}
	}
} /* CDC_out_handler */
#endif




#if(CDC_IN_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
//	-----------------------------------------------------------------------------
/**
 *  CDC_in_handler
 *  Interrupt handler 
 *  USB layer generate this function after succesfully closed DATA IN transfer
 *  This function can be also called by vcom layer when SOF is being
 *  received from the Host and no data transfer is present on DATA IN pipe
 */
 // -----------------------------------------------------------------------------
static void CDC_in_handler (USBDC_params *usbdc, USBDEP_params *ep) {
	CDC_VCOM_params *vcom = usbdc->interfaces[ep->interface].upper_layer_data.pvoid;
	uint32_t size;

	if(!vcom->in_irq || !vcom->in_buffer) {
		return;
	}
	// at first we have to check EP transfer state
	// if Tx USB pipe is not busy we can perform new transfer to Host
	if((ep->transfer_params.transfer_state == USBD_END_OF_TRANSFER)
			|| (ep->transfer_params.transfer_state == USBD_TOO_LESS_SPACE_FOR_NEXT_PACKET)
			|| ((ep->transfer_params.transfer_state == USBD_IDLE_OR_BACKGROUND) && !(ep->transfer_params.transfer_type & USBD_TRANSFER_PRESENT))) {

		size = vcom->in_irq(vcom, vcom->in_buffer, ep->max_packet_size);
		if(size) {
			USBD_send_data_in(vcom->in_buffer, size, ep->number, usbdc);
		}
	}
} /* CDC_in_handler */
#endif




// -----------------------------------------------------------------------------
//  CDC_VCOM_init
// -----------------------------------------------------------------------------
void CDC_VCOM_init (CDC_VCOM_params *vcom) {
	memset((void*)vcom, 0, sizeof(CDC_VCOM_params));
} /* CDC_VCOM_init */




// -----------------------------------------------------------------------------
//  CDC_VCOM_install
// -----------------------------------------------------------------------------
void CDC_VCOM_install (
		USBDC_params *usbdc,
		CDC_VCOM_params *vcom,
		uint8_t notification_interface,
		uint8_t data_interface,
		uint8_t ep_in_number,
		uint8_t ep_out_number,
		uint8_t ep_notif_number
		) {
	union {
		void *pvoid;
		void (*hvoid)(USBDC_params *, uint8_t, USBDE_params *);
	}owner;

	owner.hvoid = CDC_event_handler;

	// at first we have to save the vCOM pointer inside Configuration structure
	usbdc->interfaces[notification_interface].upper_layer_data.pvoid = vcom;
	usbdc->interfaces[data_interface].upper_layer_data.pvoid = vcom;
	usbdc->interfaces[notification_interface].owner = owner.pvoid;
	usbdc->interfaces[data_interface].owner = owner.pvoid;

	// then we have to fill all fields in vCOM structure
	vcom->usbdc = usbdc;
	vcom->serial_state_notification.Request.RequestFields.bmRequestType = 0xA1;
	vcom->serial_state_notification.Request.RequestFields.bRequest = 0x20;
	vcom->serial_state_notification.wIndex = notification_interface;
	vcom->serial_state_notification.wLength = 2;
	vcom->ep_in_number = ep_in_number;
	vcom->ep_out_number = ep_out_number;
	vcom->ep_notif_number = ep_notif_number;

	// and also we must connect CDC_CCI_control_handler
	// to configuration, clear Endpoints handlers
	// and connect event handler
	usbdc->ep0_setup_interface_irq[notification_interface] = CDC_CCI_control_handler;

	// by default we inform the USBD CORE that it have to
	// lock transfer on the out endpoint after finished transfer.
	// it allows to process the received data somewhere outside
	// the USB interrupt - each next packet will be NACK-ed
	// before unlock sequence (USBDEP_data_out_ready function
	// called on the out endpoint) will be performed.
	usbdc->ep_data_out_wait_for_ready_mask |= 1 << ep_out_number;

#if(CDC_IN_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
	usbdc->ep_irq[ep_in_number] = CDC_in_handler;
#else
	usbdc->ep_irq[ep_in_number] = 0;
#endif

#if(CDC_OUT_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
	usbdc->ep_irq[ep_out_number] = CDC_out_handler;
#else
	usbdc->ep_irq[ep_out_number] = 0;
#endif

	usbdc->ep_irq[ep_notif_number] = 0;
	USBDE_install(CDC_event_handler, USBDEM_SOF, usbdc);
} /* CDC_VCOM_install */




#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static const uint8_t CDC_VCOM_cci_descriptor_patern[] = {
		// Interface 0 Descriptor
		0x09,											// bLength
		USBD_DESC_TYPE_INTERFACE,						// bDescriptorType
		0,						// default				// bInterfaceNumber
		0x00,											// bAlternateSetting
		0x01,											// bNumEndpoints
		0x02,											// bInterfaceClass	Communication Class Interface
		0x02,											// bInterfaceSubclass
		0x01,											// bInterfaceProtocol
		0x00,											// iInterface

		// Functional Descriptors
		0x05,											//bFunctionLength
		0x24,											//bDescriptorType - CS_INTERFACE
		0x00,											//bDescriptorSubtype - Header
		0x10,
		0x01,											//bcdCDC

		0x04,											//bFunctionLength
		0x24,											//bDescriptorType - CS_INTERFACE
		0x02,											//bDescriptorSubtype - Abstract Control Management Functional Descriptor
		0x07,											//bmCapabilities

		0x05,											//bFunctionLength
		0x24,											//bDescriptorType - CS_INTERFACE
		0x06,											//bDescriptorSubtype - Union Descriptor Functional Descriptor
		0x00,					// default				//bMasterInterface
		0x00,					// default				//bSlaveInterface0

		0x05,											//bFunctionLength
		0x24,											//bDescriptorType - CS_INTERFACE
		0x01,											//bDescriptorSubtype - Call Management Functional Descriptor
		0x01,											//bmCapabilities
		0x00,					// default				//bDataInterface

		// Endpoint 1 descriptor
		0x07,											// bLength
		USBD_DESC_TYPE_ENDPOINT,						// bDescriptorType
		0x80,					// default				// bEndpointAddress, Endpoint IN
		0x03,											// bmAttributes      INTERRUPT
		0x00,					// default				// wMaxPacketSize
		0x00,					// default
		USBDC_CCI_EP_bInterval							// bInterval
};

static const uint8_t CDC_VCOM_cdi_descriptor_pattern[] = {
		// Interface 1 Descriptor
		0x09,											// bLength
		USBD_DESC_TYPE_INTERFACE,						// bDescriptorType
		0x00,					// default				// bInterfaceNumber
		0x00,											// bAlternateSetting
		0x02,											// bNumEndpoints
		0x0A,											// bInterfaceClass	Communication Class Interface
		0x00,											// bInterfaceSubclass
		0x00,											// bInterfaceProtocol
		0x00,											// iInterface

		// Endpoint 2 descriptor
		0x07,											// bLength
		USBD_DESC_TYPE_ENDPOINT,						// bDescriptorType
		0x80,					// default				// bEndpointAddress, Endpoint IN
		0x02,											// bmAttributes      BULK
		0x00,					// default				// wMaxPacketSize
		0x00,					// default
		USBDC_CDI_EP_IN_bInterval,						// bInterval

		// Endpoint 3 descriptor
		0x07,											// bLength
		USBD_DESC_TYPE_ENDPOINT,						// bDescriptorType
		0x00,					// default				// bEndpointAddress, Endpoint OUT
		0x02,											// bmAttributes      BULK
		0x00,					// default				// wMaxPacketSize
		0x00,					// default
		USBDC_CDI_EP_OUT_bInterval						// bInterval
};
#endif



#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static uint8_t CDC_VCOM_cci_parse(
		void **data,
		uint8_t *notif_ep,
		uint8_t config_interface,
		uint8_t *data_interface,
		CCM_session_params* session)
{
	CCM_interface_params_get interface_params;
	CCM_ep_params_get ep_params;
	CDC_VCOM_functional_descriptors *functional;
	USBD_interface_descriptor *interface;
	USBD_endpoint_descriptor *endpoint;

	// get final parameters of the Communication Class Interface
	if(CCM_FINISH_get_interface_params(&interface_params, config_interface, session)) {
		return(CCM_FINISH_ERROR);
	}

	// Communication Class Interface has only one slave interface
	// called Communication Data Interface - check it
	if(interface_params.association.master.num_slaves != 1) {
		return(CCM_FINISH_ERROR);
	}

	// gets number of a first slave interface
	if(CCM_FINISH_get_slave_interface_num_by_id(data_interface, config_interface, 0, session)) {
		return(CCM_FINISH_ERROR);
	}

	// Communication Class Interface has only one endpoint - check it
	if(interface_params.num_endpoints != 1) {
		return(CCM_FINISH_ERROR);
	}

	// gets parameters of the interrupt endpoint
	// used by the Communication Class Interface
	if(CCM_FINISH_get_ep_params(&ep_params, 0, config_interface, session)) {
		return(CCM_FINISH_ERROR);
	}

	// we have signed this endpoint during class adding
	// (in CDC_VCOM_add_class function) as 0 - now we check this identificator
	// here we also check type of this endpoint
	if((ep_params.ep_id != 0)
			|| ((ep_params.type & USBDEP_TYPE_INTERRUPT_IN) != USBDEP_TYPE_INTERRUPT_IN)) {
		return(CCM_FINISH_ERROR);
	}

	// save pointer to vCOM object
	*data = interface_params.data;

	// save physical number of this endpoint
	*notif_ep = ep_params.ep_num;

	// set pattern of the interface descriptor
	memcpy(
			interface_params.descriptor,
			CDC_VCOM_cci_descriptor_patern,
			sizeof(CDC_VCOM_cci_descriptor_patern));

	interface = (USBD_interface_descriptor*)(interface_params.descriptor);
	functional = (CDC_VCOM_functional_descriptors*)(interface_params.descriptor + sizeof(USBD_interface_descriptor));
	endpoint = (USBD_endpoint_descriptor*)(interface_params.descriptor + sizeof(USBD_interface_descriptor) + sizeof(CDC_VCOM_functional_descriptors));

	// put correct values of the specific fields of the interface descriptor
	interface->bInterfaceNumber = config_interface;
	functional->union_descriptor.bMasterInterface = config_interface;
	functional->union_descriptor.bSlaveInterface0 = *data_interface;
	functional->call_management.bDataInterface = *data_interface;
	endpoint->bEndpointAddress |= ep_params.ep_num;
	endpoint->wMaxPacketSize.L = ep_params.max_packet_size.uint8.L;
	endpoint->wMaxPacketSize.H = ep_params.max_packet_size.uint8.H;

	return(CCM_ERROR_NONE);
}
#endif



#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static uint8_t CDC_VCOM_cdi_parse(
		uint8_t *in_ep,
		uint8_t *out_ep,
		uint8_t data_interface,
		CCM_session_params* session) {
	CCM_interface_params_get interface_params;
	CCM_ep_params_get ep_params;
	USBD_interface_descriptor *interface;
	USBD_endpoint_descriptor *endpoint;
	uint8_t counter;

	// now we gets informations of a slave interface - Data Class Interface
	if(CCM_FINISH_get_interface_params(&interface_params, data_interface, session)) {
		return(CCM_FINISH_ERROR);
	}

	// Communication Data Interface has no slave interfaces - check it
	if(interface_params.association.master.num_slaves != 0) {
		return(CCM_FINISH_ERROR);
	}

	// Communication Data Interface has only two endpoints - check it
	if(interface_params.num_endpoints != 2) {
		return(CCM_FINISH_ERROR);
	}

	// set pattern of the interface descriptor
	memcpy(
			interface_params.descriptor,
			CDC_VCOM_cdi_descriptor_pattern,
			sizeof(CDC_VCOM_cdi_descriptor_pattern));

	interface = (USBD_interface_descriptor*)(interface_params.descriptor);

	// put correct value of the interface number in the interface descriptor
	interface->bInterfaceNumber = data_interface;


	for(counter = 0; counter < 2; counter++) {
		// gets parameters of a bulk endpoint
		// used by the Communication Data Interface
		if(CCM_FINISH_get_ep_params(&ep_params, counter, data_interface, session)) {
			return(CCM_FINISH_ERROR);
		}

		if((ep_params.type & USBDEP_TYPE_BULK_IN) == USBDEP_TYPE_BULK_IN) {
			endpoint = (USBD_endpoint_descriptor*)(interface_params.descriptor + sizeof(USBD_interface_descriptor));

			if(*in_ep) {
				return(CCM_FINISH_ERROR);
			}

			*in_ep = ep_params.ep_num;
			endpoint->bEndpointAddress |= ep_params.ep_num;
			endpoint->wMaxPacketSize.L = ep_params.max_packet_size.uint8.L;
			endpoint->wMaxPacketSize.H = ep_params.max_packet_size.uint8.H;

		}
		else if((ep_params.type & USBDEP_TYPE_BULK_OUT) == USBDEP_TYPE_BULK_OUT) {
			endpoint = (USBD_endpoint_descriptor*)(interface_params.descriptor + sizeof(USBD_interface_descriptor) + sizeof(USBD_endpoint_descriptor));

			if(*out_ep) {
				return(CCM_FINISH_ERROR);
			}

			*out_ep = ep_params.ep_num;
			endpoint->bEndpointAddress |= ep_params.ep_num;
			endpoint->wMaxPacketSize.L = ep_params.max_packet_size.uint8.L;
			endpoint->wMaxPacketSize.H = ep_params.max_packet_size.uint8.H;
		}
	}

	return(CCM_ERROR_NONE);
}
#endif



#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
static uint8_t CDC_VCOM_add_finish(USBDC_params *usbdc, uint8_t interface, CCM_session_params* session) {
	void *data;
	uint8_t notif_ep;
	uint8_t in_ep = 0;
	uint8_t out_ep = 0;
	uint8_t config_interface = interface;
	uint8_t data_interface;
	uint8_t result;

	// parses Communication Class Interface
	result = CDC_VCOM_cci_parse(&data, &notif_ep, config_interface, &data_interface, session);
	if(result) {
		return(result);
	}

	// parses Communication Data Interface
	result = CDC_VCOM_cdi_parse(&in_ep, &out_ep, data_interface, session);
	if(result) {
		return(result);
	}

	CDC_VCOM_init (data);
	CDC_VCOM_install(usbdc, data, config_interface, data_interface, in_ep, out_ep, notif_ep);

	return(CCM_ERROR_NONE);
}
#endif




#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
uint8_t CDC_VCOM_add_item(CDC_VCOM_params *vcom, CCM_session_params* session) {
	uint8_t result;
	union {
		uint8_t interface;
		uint16_t ret;
		uint8_t uint8_tab[2];
	}class_interface, data_interface;
	CCM_ep_params_add ep_params;

	// ----------------------------------------------------------------------
	// Communication Class Interface
	// ----------------------------------------------------------------------

	// add Communication Class Interface
	class_interface.ret = CCM_add_interface(
			vcom,
			CDC_VCOM_add_finish,
			sizeof(USBD_interface_descriptor) + sizeof(USBD_endpoint_descriptor) + sizeof(CDC_VCOM_functional_descriptors),
			session);

	// try result
	if(class_interface.ret >= USBD_MAX_NUM_INTERFACES) {
		return(class_interface.uint8_tab[1]);
	}

	// --------------------------------------------------
	// Notification Endpoint
	// --------------------------------------------------

	// prepare parameters of this endpoint
	ep_params.ep_id = 0;
	ep_params.type = USBDEP_TYPE_INTERRUPT_IN;
	ep_params.max_packet_size.uint16 = 16;
	ep_params.max_packet_size_priority = 150;
	ep_params.interface_num = class_interface.interface;

	// add INTERRUPT_IN endpoint
	result = CCM_add_ep(&ep_params, session);

	// ep_add failed
	if(result) {
		CCM_remove_last_interface(session);
		return(result);
	}

	// ----------------------------------------------------------------------
	// Communication Data Interface
	// ----------------------------------------------------------------------

	// add Communication Data Interface
	data_interface.ret = CCM_add_interface(
			vcom,
			CDC_VCOM_add_finish,
			sizeof(USBD_interface_descriptor) + (2* sizeof(USBD_endpoint_descriptor)),
			session);

	// try result
	if(data_interface.ret >= USBD_MAX_NUM_INTERFACES) {
		CCM_remove_last_interface(session);
		return(data_interface.uint8_tab[1]);
	}

	CCM_associate_interface(data_interface.interface, class_interface.interface, 0, session);

	// --------------------------------------------------
	// Bulk In Endpoint
	// --------------------------------------------------

	// prepare parameters of this endpoint
	ep_params.ep_id = 1;
	ep_params.type = USBDEP_TYPE_BULK_IN;
	ep_params.max_packet_size.uint16 = USBDEP_HS_BULK_MAX_POSSIBLE_PACKET_SIZE;
	ep_params.max_packet_size_priority = 10;
	ep_params.interface_num = data_interface.interface;

	// add BULK_IN endpoint
	result = CCM_add_ep(&ep_params, session);

	// ep_add failed
	if(result) {
		CCM_remove_last_interface(session);
		CCM_remove_last_interface(session);
		return(result);
	}

	// --------------------------------------------------
	// Bulk Out Endpoint
	// --------------------------------------------------

	// prepare parameters of this endpoint
	ep_params.ep_id = 2;
	ep_params.type = USBDEP_TYPE_BULK_OUT;
	// other params - same like for Bulk In Endpoint

	// add BULK_OUT endpoint
	result = CCM_add_ep(&ep_params, session);

	// ep_add failed
	if(result) {
		CCM_remove_last_interface(session);
		CCM_remove_last_interface(session);
		return(result);
	}

	// vcom reserved successfully
	return(CCM_ERROR_NONE);
}
#endif



/*@}*/

/*! \file cdc_vcom.c
    \brief vCOM library
*/
