/*
================================================================================
 File: cdc_vcom.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USB virtual serial port based on USB CDC ACM device class
 Compiler:      ANSI C
 Hardware:      
 Doc:           
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef CDC_VCOM_H_
#define CDC_VCOM_H_

#ifndef USBD_H_
#include "usbd.h"
#endif

/** \defgroup vCOM virtual COM library

@{*/



//  *****************************************************************************
//				USB section
//  *****************************************************************************

typedef enum {
USBD_CDC_BREQUEST_SEND_ENCAPSULATED_COMMAND		= 0x00,
USBD_CDC_BREQUEST_GET_ENCAPSULATED_RESPONSE		= 0x01,

USBD_CDC_BREQUEST_SET_COMM_FEATURE				= 0x02,
USBD_CDC_BREQUEST_GET_COMM_FEATURE				= 0x03,
USBD_CDC_BREQUEST_CLEAR_COMM_FEATURE			= 0x04,

USBD_CDC_BREQUEST_SET_LINE_CODING				= 0x20,
USBD_CDC_BREQUEST_GET_LINE_CODING				= 0x21,

USBD_CDC_BREQUEST_SET_CONTROL_LINE_STATE		= 0x22,

USBD_CDC_BREQUEST_SEND_BREAK					= 0x23
}USBDR_bRequest_CDC_ACM_types;



typedef enum {
CDC_SEND_ENCAPSULATED_COMMAND_INTERFACE			= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_SEND_ENCAPSULATED_COMMAND << 8)),
CDC_GET_ENCAPSULATED_RESPONSE_INTERFACE			= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_GET_ENCAPSULATED_RESPONSE << 8)),

CDC_SET_COMM_FEATURE_INTERFACE   				= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_SET_COMM_FEATURE << 8)),
CDC_GET_COMM_FEATURE_INTERFACE   				= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_GET_COMM_FEATURE << 8)),
CDC_CLEAR_COMM_FEATURE_INTERFACE 				= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_CLEAR_COMM_FEATURE << 8)),

CDC_SET_LINE_CODING_INTERFACE    				= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_SET_LINE_CODING << 8)),
CDC_GET_LINE_CODING_INTERFACE    				= (USBD_BMREQUESTTYPE_GET_SYNCH_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_GET_LINE_CODING << 8)),

CDC_SET_CONTROL_LINE_STATE_INTERFACE			= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_SET_CONTROL_LINE_STATE << 8)),

CDC_SEND_BREAK_INTERFACE						= (USBD_BMREQUESTTYPE_SET_CLEAR_INTERFACE | USBD_BMREQUESTTYPE_CLASS | (USBD_CDC_BREQUEST_SEND_BREAK << 8))
} USBDR_CDC_ACM_types;






//  *****************************************************************************
//				virtual COM section
//  *****************************************************************************


//				ATTENTION !!!
//	direction of TxD signal is from vCOM library to Computer and direction of RxD signal is from Computer to vCOM library


/**
 * If this definition is different than USBD_FEATURE_OFF and if we do not
 * declare DTR handler then CDC will automatically respond to Host
 * after change DTR by changing DSR signal. 
 */
#ifndef CDC_DTR_AUTO_REPEAT_SUPPORT
#define CDC_DTR_AUTO_REPEAT_SUPPORT			USBD_FEATURE_ON
#endif
/**
 * If this definition is different than USBD_FEATURE_OFF and if we do not
 * declare RTS handler then CDC will automatically respond to Host
 * after change DTR by changing DCD signal. 
 */
#ifndef CDC_RTS_AUTO_REPEAT_SUPPORT
#define CDC_RTS_AUTO_REPEAT_SUPPORT			USBD_FEATURE_ON
#endif
/**
 * If this definition is different than USBD_FEATURE_OFF then this
 * library uses its own function for OUT DATA endpoint
 */
#ifndef CDC_IN_DATA_STREAM_SUPPORT
#define CDC_IN_DATA_STREAM_SUPPORT			USBD_FEATURE_ON
#endif
/**
 * If this definition is different than USBD_FEATURE_OFF then this
 * library uses its own function for IN DATA endpoint
 */
#ifndef CDC_OUT_DATA_STREAM_SUPPORT
#define CDC_OUT_DATA_STREAM_SUPPORT			USBD_FEATURE_ON
#endif
/**
 * If this definition is different than USBD_FEATURE_OFF then there is
 * special parameter inside CDC_VCOM_params structure used by upper layer
 */
#ifndef CDC_UPPER_LAYER_SUPPORT
#define CDC_UPPER_LAYER_SUPPORT				USBD_FEATURE_ON
#endif

#ifndef USBDC_CCI_EP_bInterval
#define USBDC_CCI_EP_bInterval				1
#endif

#ifndef USBDC_CDI_EP_IN_bInterval
#define USBDC_CDI_EP_IN_bInterval			10
#endif

#ifndef USBDC_CDI_EP_OUT_bInterval
#define USBDC_CDI_EP_OUT_bInterval			10
#endif



typedef struct {
	struct {
		uint8_t bFunctionLength;
		uint8_t bDescriptorType;
		uint8_t bDescriptorSubtype;
		uint8_t bcdCDC_L;
		uint8_t bcdCDC_H;
	}header;
	struct {
		uint8_t bFunctionLength;
		uint8_t bDescriptorType;
		uint8_t bDescriptorSubtype;
		uint8_t bmCapabilities;
	}abstract_control_management;
	struct {
		uint8_t bFunctionLength;
		uint8_t bDescriptorType;
		uint8_t bDescriptorSubtype;
		uint8_t bMasterInterface;
		uint8_t bSlaveInterface0;
	}union_descriptor;
	struct {
		uint8_t bFunctionLength;
		uint8_t bDescriptorType;
		uint8_t bDescriptorSubtype;
		uint8_t bmCapabilities;
		uint8_t bDataInterface;
	}call_management;
}CDC_VCOM_functional_descriptors;




/**
 * This structure stores serial port transmition parameters
 */
typedef struct {
	uint32_t baudrate;
	uint8_t stop_bits;
	uint8_t partity;
	uint8_t data_bits;
}CDC_VCOM_line_coding;




/**
 * This structure stores all virtual COM necessary parameters 
 */
struct _CDC_VCOM_params {
	USBDC_params *usbdc;
	
	uint16_t abstract_state;
	
	uint16_t control_line_state;
	
	uint16_t country_setting;
	
	uint16_t duration_of_break;
	
	USBDR_params serial_state_notification;
	
	uint16_t serial_state_notification_data;
	
	CDC_VCOM_line_coding line_coding;
	
	uint8_t ep_in_number;
	
	uint8_t ep_out_number;
	
	uint8_t ep_notif_number;
	
	void (*change_params_irq) (struct _CDC_VCOM_params *vcom);
	
	void (*rts_irq) (struct _CDC_VCOM_params *vcom);
	
	void (*dtr_irq) (struct _CDC_VCOM_params *vcom);
	
#if(CDC_IN_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
	uint32_t (*in_irq)(struct _CDC_VCOM_params *vcom, uint8_t *buffer, uint32_t size);
	
	uint8_t *in_buffer;
#endif
	
#if(CDC_OUT_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
	// if this function returns 0 as result of finished processing data
	// otherwise endpoint will be still locked for receiving
	// next data packets. Then we have to use CDC_VCOM_RxD_DATA_ready
	// macro to unlock OUT transfer
	uint32_t (*out_irq)(struct _CDC_VCOM_params *vcom, uint8_t* buffer, uint32_t size);

	// buffer for OUT operations
	// this buffer is this same buffer as parameter
	// of OUT handler. It has to have size equal to
	// OUT Endpoint max packet size
	uint8_t *out_buffer;
#endif

#if(CDC_UPPER_LAYER_SUPPORT	== USBD_FEATURE_ON)
	// parameter used by upper layer
	union {
		void *pvoid;
		uint32_t uint32;
		uint16_t uint16;
		uint8_t uint8;
	}upper_layer_data;
#endif
};
typedef struct _CDC_VCOM_params CDC_VCOM_params;


/**
 * Initializes declared vCOM by clear all its fields
 */
void CDC_VCOM_init (CDC_VCOM_params *vcom);


/**
 * This function fill all requied fields in USBDC_params *usbdc
 * and CDC_VCOM_params *vcom structures and install requied functions
 * in *usbdc structure. To install vCOM in USB configuration
 * user is obliged to create USB Configuration Descriptor that includes
 * two CDC interfaces: Communication Class Interface
 * and Communication Data Dnterface.
 * 
 * Communication Class Interface should provide 6 descriptors:
 * 		- Interface Descriptor (its own descriptor)
 * 		- Header Descriptor
 * 		- Abstract Control Management Functional Descriptor
 * 		- Union Descriptor Functional Descriptor
 * 		- Call Management Functional Descriptor
 * 		- Endpoint NOTIF Descriptor
 * 
 * Communication Data Interface should provide 3 descriptors:
 * 		- Interface Descriptor (its own descriptor)
 * 		- Endpoint IN Descriptor
 * 		- Endpoint OUT Descriptor
 * 
 * Calling CDC_VCOM_install function user have to give following parameters:
 * 		- pointer to configuration
 * 		- pointer to initialized vCOM
 * 		- number of Communication Class Interface - NOTIF_Interface
 * 		- number of Communication Data Interface - DATA_Intercase
 * 		- number of IN Endpoint
 * 		- number of OUT Endpoint
 * 		- number of NOTIF Endpoint
 * 
 *  For more informations about USB CDC class please refer
 *                  to USB CDC manual
 * 
 * Using this function is mandatory.
 */
void CDC_VCOM_install (
		USBDC_params *usbdc,
		CDC_VCOM_params *vcom,
		uint8_t notification_interface,
		uint8_t data_interface,
		uint8_t ep_in_number,
		uint8_t ep_out_number,
		uint8_t ep_notif_number
		);


#if(USBD_CCM_SUPPORT == USBD_FEATURE_ON)
uint8_t CDC_VCOM_add_item(CDC_VCOM_params *vcom, CCM_session_params* session);
#endif


/**
 * Connect handler to function that is called
 * when Host set new parameters of virtual COM.
 */
#define CDC_VCOM_connect_change_params_handler(vcom, handler) \
	(vcom)->change_params_irq = (handler)
/**
 * Disconnects handler to function that is called
 * when Host set new parameters of virtual COM.
 */
#define CDC_VCOM_disconnect_change_params_handler(vcom) \
	(vcom)->change_params_irq = 0


/**
 * Connects handler to function that is called
 * when Host change state of RTS signal.
 */
#define CDC_VCOM_connect_RTS_handler(vcom, handler) \
	(vcom)->rts_irq = (handler)
/**
 * Disconnects handler to function that is called
 * when Host change state of RTS signal.
 */
#define CDC_VCOM_disconnect_RTS_handler(vcom) \
	(vcom)->rts_irq = 0


/**
 * Connects handler to function that is called
 * when Host change state of DTR signal.
 */
#define CDC_VCOM_connect_DTR_handler(vcom, handler) \
	(vcom)->dtr_irq = (handler)
/**
 * Disconnects handler to function that is called
 * when Host change state of DTR signal.
 */
#define CDC_VCOM_disconnect_DTR_handler(vcom) \
	(vcom)->dtr_irq = 0


/**
 * Connects handler to function that is called
 * when data from Host was succesfully received.
 * Function that we are connecting is called directly by USB layer.
 */
#define CDC_VCOM_connect_usbdep_out_handler(vcom, handler) \
	(vcom)->Conf->ep_irq[(vcom)->ep_out_number] = (handler)
/**
 * Disconnect handler to function that is called
 * when data from Host was succesfully received.
 */
#define CDC_VCOM_disconnect_usbdep_out_handler(vcom) \
	(vcom)->Conf->ep_irq[(vcom)->ep_out_number] = 0


/**
 * Connects handler to function that is called
 * when data to Host was succesfully transmitted.
 * Function that we are connecting is called directly by USB layer.
 */
#define CDC_VCOM_connect_usbdep_in_handler(vcom, handler) \
	(vcom)->Conf->ep_irq[(vcom)->ep_in_number] = (handler)
/**
 * Disconnects handler to function that is called
 * when data to Host was succesfully transmitted.
 */
#define CDC_VCOM_disconnect_usbdep_in_handler(vcom) \
	(vcom)->Conf->ep_irq[(vcom)->ep_in_number] = 0




#if(CDC_OUT_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
/**
 * connects handler to function called when received data
 * has been succesfully written to buffer and is ready
 * to be processed
 */
#define CDC_VCOM_connect_RxD_handler(vcom, handler) \
	(vcom)->out_irq = (handler)
/**
 * disconnects RxD_handler
 */
#define CDC_VCOM_disconnect_RxD_handler(vcom) \
	(vcom)->out_irq = 0
/**
 * connects RxD buffer that is used to store OUT data
 */
#define CDC_VCOM_connect_RxD_buffer(vcom, buffer) \
	(vcom)->out_buffer = (buffer)
/**
 * disconnects RxD_buffer
 */
#define CDC_VCOM_disconnect_RxD_buffer(vcom) \
	(vcom)->out_buffer = 0
/**
 * this function is used when OUT handler returns
 * value not equal to zero. Then upper layer
 * has to have indicate in some time later that has finished
 * processing data by calling this macro.
 * by using this macro we unlock endpoint for receiving
 * next packets of data
 */
#define CDC_VCOM_RxD_data_ready(vcom) \
	USBDEP_data_out_ready((vcom)->ep_out_number, (vcom)->usbdc)


#endif



#if(CDC_IN_DATA_STREAM_SUPPORT == USBD_FEATURE_ON)
/**
 * connects handler to function called when transmitted data
 * has been succesfully sent to host and we can perform next
 * packet of data
 */
#define CDC_VCOM_connect_TxD_handler(vcom, handler) \
	(vcom)->in_irq = (handler)
/**
 * disconnects TxD_handler
 */
#define CDC_VCOM_disconnect_TxD_handler(vcom) \
	(vcom)->in_irq = 0
/**
 * connects RxD buffer that is used to store IN data
 */
#define CDC_VCOM_connect_TxD_buffer(vcom, buffer) \
	(vcom)->in_buffer = (buffer)
/**
 * disconnects TxD_buffer
 */
#define CDC_VCOM_disconnect_TxD_buffer(vcom) \
	(vcom)->in_buffer = 0

#endif


#if(CDC_UPPER_LAYER_SUPPORT	== USBD_FEATURE_ON)
/**
 * sets upper layer pvoid-type data
 */
#define CDC_VCOM_set_upper_layer_pvoid(vcom, pvoid) \
	(vcom)->upper_layer_data.pvoid = ((void*)(pvoid))
/**
 * sets upper layer uint32_t-type data
 */
#define CDC_VCOM_set_upper_layer_uint32(vcom, uint32) \
	(vcom)->upper_layer_data.uint32 = ((uint32_t)(uint32))
/**
 * sets upper layer uint16_t-type data
 */
#define CDC_VCOM_set_upper_layer_uint16(vcom, uint16) \
	(vcom)->upper_layer_data.uint16 = ((uint16_t)(uint16))
/**
 * sets upper layer uint8_t-type data
 */
#define CDC_VCOM_set_upper_layer_uint8(vcom, uint8) \
	(vcom)->upper_layer_data.uint8 = ((uint8_t)(uint8))


/**
 * gets upper layer pvoid-type data
 */
#define CDC_VCOM_get_upper_layer_pvoid(vcom) \
	((void*)((vcom)->upper_layer_data.pvoid))
/**
 * gets upper layer uint32_t-type data
 */
#define CDC_VCOM_get_upper_layer_uint32(vcom) \
	((uint32_t)((vcom)->upper_layer_data.uint32))
/**
 * gets upper layer uint16_t-type data
 */
#define CDC_VCOM_get_upper_layer_uint16(vcom) \
	((uint16_t)((vcom)->upper_layer_data.uint16))
/**
 * gets upper layer uint8_t-type data
 */
#define CDC_VCOM_get_upper_layer_uint8(vcom) \
	((uint8_t)((vcom)->upper_layer_data.uint8))

#endif





/**
 * Returns vCOM baudrate. This macro returns 32-bits value.
 */
#define CDC_VCOM_get_baudrate(vcom) 	((vcom)->line_coding.baudrate)
/**
 * Returns vCOM stop bits. This macro returns 8-bits value.
 */
#define CDC_VCOM_get_stop_bits(vcom) 	((vcom)->line_coding.stop_bits)
/**
 * Returns vCOM parity. This macro returns 8-bits value.
 */
#define CDC_VCOM_get_partity(vcom) 	((vcom)->line_coding.partity)
/**
 * Returns vCOM data bits. This macro returns 8-bits value.
 */
#define CDC_VCOM_get_data_bits(vcom) 	((vcom)->line_coding.data_bits)


/**
 * Returns vCOM state of RTS signal. This macro returns 16-bits value
 * however returns just 0 or 1.
 */
#define CDC_VCOM_get_RTS(vcom) 			(((vcom)->control_line_state & 0x2) >> 1)
/**
 * Returns vCOM state of DTR signal. This macro returns 16-bits value
 * however returns just 0 or 1.
 */
#define CDC_VCOM_get_DTR(vcom) 			(((vcom)->control_line_state & 0x1) >> 0)



/**
 * Sets vCOM DSR signal.
 */
#define CDC_VCOM_set_DSR(vcom) { (vcom)->serial_state_notification_data |= 0x2; }
/**
 * Resets vCOM DSR signal.
 */
#define CDC_VCOM_clr_DSR(vcom) { (vcom)->serial_state_notification_data &= ~((uint32_t) 0x2); }
/**
 * Returns vCOM state of DSR signal. This macro returns 16-bits value
 * however returns just 0 or 1.
 */
#define CDC_VCOM_get_DSR(vcom) (((vcom)->serial_state_notification_data & 0x02) >>1)


/**
 * Sets vCOM DCD signal.
 */
#define CDC_VCOM_set_DCD(vcom) { (vcom)->serial_state_notification_data |= 0x1; }
/**
 * Resets vCOM DCD signal.
 */
#define CDC_VCOM_clr_DCD(vcom) { (vcom)->serial_state_notification_data &= ~((uint32_t) 0x1); }
/**
 * Returns vCOM state of DCD signal. This macro returns 16-bits value
 * however returns just 0 or 1.
 */
#define CDC_VCOM_get_DCD(vcom) ((vcom)->serial_state_notification_data & 0x01)





/**
 * Sets vCOM DSR signal and sends this information to Host.
 */
#define CDC_VCOM_open_port(vcom) {\
	if(!get_DSR((vcom))) {\
		set_DSR((vcom));\
		USBD_send_data_in((uint8_t*)(vcom)->serial_state_notification, 10, (vcom)->ep_notif_number, (vcom)->usbdc);\
	}\
}

/**
 * Resets vCOM DSR signal and sends this information to Host.
 */
#define CDC_VCOM_close_port(vcom) {\
	if(get_DSR((vcom))) {\
		clr_DSR((vcom));\
		USBD_send_data_in((uint8_t*)(vcom)->serial_state_notification, 10, (vcom)->ep_notif_number, (vcom)->usbdc);\
	}\
}


/**
 * Sets vCOM DCD signal and sends this information to Host.
 */
#define CDC_VCOM_open_TxD(vcom) {\
	if(!get_DCD((vcom))) {\
		set_DCD((vcom));\
		USBD_send_data_in((uint8_t*)(vcom)->serial_state_notification, 10, (vcom)->ep_notif_number, (vcom)->usbdc);\
	}\
}

/**
 * Resets vCOM DCD signal and sends this information to Host.
 */
#define CDC_VCOM_close_TxD(vcom) {\
	if(get_DCD((vcom))) {\
		clr_DCD((vcom));\
		USBD_send_data_in((uint8_t*)(vcom)->serial_state_notification, 10, (vcom)->ep_notif_number, (vcom)->usbdc);\
	}\
}



/*@}*/

/*! \file cdc_virtual_com.h
    \brief vCOM library
*/

#endif /* cdc_vcom_h */
