/*
================================================================================
 File: usbd_cdp.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD configuration descriptor parser
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_CDP_H_
#include "usbd_cdp.h"
#endif

#ifndef USBD_H_
#include "usbd.h"
#endif


#if((USBD_MAX_NUM_ENDPOINTS > 1) || (USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (USBD_UPPER_LAYER_SUPPORT == USBD_FEATURE_ON) || (!USBD_CDP_USED_JUST_BY_USBD_CORE))
uint16_t USBDCD_find_interface_address (uint8_t *descriptor, uint8_t interface_number, uint8_t alternate_setting) {
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}elems;
	}total_length;
	uint16_t position = 0;

	total_length.elems.L = descriptor[2];
	total_length.elems.H = descriptor[3];
	do {
		if(descriptor[position + 1] == USBD_DESC_TYPE_INTERFACE) {
			if((descriptor[position + 2] == interface_number) && (descriptor[position + 3] == alternate_setting)) {
				return (position);
			} 
		}
		position += descriptor[position];
	} while (position < total_length.uint16);
	return (0);
}

#if((USBD_ALTERNATE_SETTING_SUPPORT == USBD_FEATURE_ON) || (!USBD_CDP_USED_JUST_BY_USBD_CORE))
uint8_t USBDCD_find_interface_max_alternate_setting (uint8_t *descriptor, uint8_t interface_number) {
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}elems;
	}total_length;
	uint16_t position = 0;
	uint8_t alternate_setting = 0;

	total_length.elems.L = descriptor[2];
	total_length.elems.H = descriptor[3];
	do {
		if(descriptor[position + 1] == USBD_DESC_TYPE_INTERFACE) {
			if(descriptor[position + 2] == interface_number) {
				alternate_setting++;
			} 
		}
		position += descriptor[position];
	} while (position < total_length.uint16);
	return (alternate_setting);
}
#endif

#if((USBD_MAX_NUM_ENDPOINTS > 1) || (!USBD_CDP_USED_JUST_BY_USBD_CORE))
uint16_t USBDCD_find_next_endpoint_address (uint8_t *descriptor, uint16_t start_address) {
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}elems;
	}total_length;
	uint16_t position = start_address;

	total_length.elems.L = descriptor[2];
	total_length.elems.H = descriptor[3];
	do {
		position += descriptor[position];
		if(descriptor[position + 1] == USBD_DESC_TYPE_ENDPOINT) {
			return (position);
		}
	} while ((position < total_length.uint16) && (descriptor[position + 1] != USBD_DESC_TYPE_INTERFACE));
	return (0);
}

void USBDCD_create_ep_max_used_packet_size_table (uint8_t *descriptor, uint16_t *table) {
	union {
		uint16_t uint16;
		struct {
			uint8_t L;
			uint8_t H;
		}elems;
	}total_length, max_packet_size;
	uint16_t position = 0;
	uint8_t ep_number;

	total_length.elems.L = descriptor[2];
	total_length.elems.H = descriptor[3];

	for(ep_number = 0; ep_number < USBD_MAX_NUM_ENDPOINTS; ep_number++) {
		table[ep_number] = 0;
	}

	do {
		if(descriptor[position + 1] == USBD_DESC_TYPE_ENDPOINT) {
			ep_number = descriptor[position + 2] & 0x0F;
			if(ep_number < USBD_MAX_NUM_ENDPOINTS) {
				max_packet_size.elems.L = descriptor[position + 4];
				max_packet_size.elems.H = descriptor[position + 5] & 0x07;
				if(table[ep_number] < max_packet_size.uint16) {
					table[ep_number] = max_packet_size.uint16;
				}
			}
		}
		position += descriptor[position];
	} while (position < total_length.uint16);
}
#endif
#endif
