/*
================================================================================
 File: usbd_cdp.h
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD configuration descriptor parser
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_CDP_H_
#define USBD_CDP_H_
#include <stdint.h>

#ifndef USBD_CONFIG_H_
#include "usbd_config.h"
#endif


#ifndef USBD_CDP_USED_JUST_BY_USBD_CORE
#define USBD_CDP_USED_JUST_BY_USBD_CORE		0
#endif


/* *****************************************************************************
				STANDARD DESCRIPTORS TYPES
   ***************************************************************************** */
#define USBD_DESC_TYPE_DEVICE           1
#define USBD_DESC_TYPE_CONFIGURATION    2
#define USBD_DESC_TYPE_STRING           3
#define USBD_DESC_TYPE_INTERFACE        4
#define USBD_DESC_TYPE_ENDPOINT         5

/* *****************************************************************************
				ENDPOINT DESCRIPTOR TRANSFER TYPES
   ***************************************************************************** */
#define USBDEP_DESC_TRANSFER_TYPE_CONTROL	    0
#define USBDEP_DESC_TRANSFER_TYPE_ISOCHRONOUS   1
#define USBDEP_DESC_TRANSFER_TYPE_BULK          2
#define USBDEP_DESC_TRANSFER_TYPE_INTERRUPT     3

/* *****************************************************************************
				ENDPOINT DESCRIPTOR DIRECTION TYPES
   ***************************************************************************** */
#define USBDEP_DESC_DIR_IN      0x80
#define USBDEP_DESC_DIR_OUT 	0x00

typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint16_t bcdUSB;
	uint8_t bDeviceClass;
	uint8_t bDeviceSubclass;
	uint8_t bDeviceProtocol;
	uint8_t bMaxPacketSize0;
	uint16_t idVendor;
	uint16_t idProduct;
	uint16_t bcdDevice;
	uint8_t iManufacturer;
	uint8_t iProduct;
	uint8_t iSerialNumber;
	uint8_t bNumConfigurations;
}USBD_device_descriptor;

typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	struct {
		uint8_t L;
		uint8_t H;
	} wTotalLength;
	uint8_t bNumInterfaces;
	uint8_t bConfigurationValue;
	uint8_t iConfiguration;
	uint8_t bmAttributes;
	uint8_t bMaxPower;
}USBD_configuration_descriptor;

typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bInterfaceNumber;
	uint8_t bAlternateSetting;
	uint8_t bNumEndpoints;
	uint8_t bInterfaceClass;
	uint8_t bInterfaceSubClass;
	uint8_t bInterfaceProtocol;
	uint8_t iInterface;
}USBD_interface_descriptor;

typedef struct {
	uint8_t bLength;
	uint8_t bDescriptorType;
	uint8_t bEndpointAddress;
	uint8_t bmAttributes;
	struct {
		uint8_t L;
		uint8_t H;
	}wMaxPacketSize;
	uint8_t bInterval;
}USBD_endpoint_descriptor;




uint16_t USBDCD_find_interface_address (
		uint8_t *descriptor,
		uint8_t interface_number,
		uint8_t alternate_setting);

uint8_t USBDCD_find_interface_max_alternate_setting (
		uint8_t *descriptor,
		uint8_t interface_number);

uint16_t USBDCD_find_next_endpoint_address (
		uint8_t *descriptor,
		uint16_t start_address);

void USBDCD_create_ep_max_used_packet_size_table (
		uint8_t *descriptor,
		uint16_t *table);

#endif /*USBD_CCM_H_*/
