/*
================================================================================
 File: usbd_port_trash.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD trash
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_PORT_TRASH_H_
#include "usbd_port_trash.h"
#endif

#ifndef USBD_STM32_HARDWARE_H_
#include "usbd_stm32_hardware.h"
#endif

#include "stm32f10x_conf.h"


// default configuration - Olimex board compatible

// GPIO: DP - enable line
#ifndef USBD_DP_PULL_UP_PORT
#define USBD_DP_PULL_UP_PORT			GPIOC
#endif

#ifndef USBD_DP_PULL_UP_PIN
#define USBD_DP_PULL_UP_PIN				GPIO_Pin_11
#endif

#ifndef USBD_DP_PULL_UP_MODE
#define USBD_DP_PULL_UP_MODE			GPIO_Mode_Out_PP
#endif

#ifndef USBD_DP_PULL_UP_ACTIVE_STATE
#define USBD_DP_PULL_UP_ACTIVE_STATE	0
#endif


// GPIO: VBUS
#ifndef USBD_VBUS_PORT
#define USBD_VBUS_PORT					GPIOC
#endif

#ifndef USBD_VBUS_PIN
#define USBD_VBUS_PIN					GPIO_Pin_4
#endif

#ifndef USBD_VBUS_MODE
#define USBD_VBUS_MODE					GPIO_Mode_IN_FLOATING
#endif

#ifndef USBD_VBUS_ACTIVE_STATE
#define USBD_VBUS_ACTIVE_STATE			1
#endif


// EXTI: VBUS
#ifndef USBD_VBUS_GPIO_PortSource
#define USBD_VBUS_GPIO_PortSource		GPIO_PortSourceGPIOC
#endif

#ifndef USBD_VBUS_GPIO_PinSource
#define USBD_VBUS_GPIO_PinSource		GPIO_PinSource4
#endif

#ifndef USBD_VBUS_EXTI_Line
#define USBD_VBUS_EXTI_Line				EXTI_Line4
#endif

#ifndef USBD_VBUS_EXTI_IRQChannel
#define USBD_VBUS_EXTI_IRQChannel		EXTI4_IRQn
#endif


// RCC: RCC_USBCLKSource_PLLCLK prescaller
#ifndef USBD_RCC_USBCLKSource_PLLCLK_presc
#define USBD_RCC_USBCLKSource_PLLCLK_presc		RCC_USBCLKSource_PLLCLK_Div1
//#define USBD_RCC_USBCLKSource_PLLCLK_presc		RCC_USBCLKSource_PLLCLK_1Div5
#endif



// from stm32f10x_rcc.c
/* --- CFGR Register ---*/
/* Alias word address of USBPRE bit */
#define RCC_OFFSET                (RCC_BASE - PERIPH_BASE)
#define CFGR_OFFSET               (RCC_OFFSET + 0x04)
#define USBPRE_BitNumber          0x16
#define CFGR_USBPRE_BB            (PERIPH_BB_BASE + (CFGR_OFFSET * 32) + (USBPRE_BitNumber * 4))



/****************************************************************************/
/*               USBD_DP_PULL_UP                                            */
/****************************************************************************/

#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
/** 
 *  Konfiguracja pinu DP_PULL_UP
*/
static void USBD_DP_PULL_UP_configure_pin(void) {
	GPIO_InitTypeDef gpio_data;

	// inicjalizacja portu linii sterujacej
	// rezystorem Pull-Up na linii D+
	gpio_data.GPIO_Speed = GPIO_Speed_50MHz;
	gpio_data.GPIO_Pin = USBD_DP_PULL_UP_PIN;
	gpio_data.GPIO_Mode = USBD_DP_PULL_UP_MODE;
	GPIO_Init(USBD_DP_PULL_UP_PORT, &gpio_data);

	// deaktywacja rezystora D+
	USBD_DP_PULL_UP_disable();
}

/** 
 *  Ustawienie pinu DP_PULL_UP w stan niski
*/
void USBD_DP_PULL_UP_enable(void) {
#if(USBD_DP_PULL_UP_ACTIVE_STATE)
	USBD_DP_PULL_UP_PORT->BSRR = USBD_DP_PULL_UP_PIN;
#else
	USBD_DP_PULL_UP_PORT->BRR = USBD_DP_PULL_UP_PIN;
#endif
}

/** 
 *  Ustawienie pinu DP_PULL_UP w stan wysoki
*/
void USBD_DP_PULL_UP_disable(void) {
#if(USBD_DP_PULL_UP_ACTIVE_STATE)
	USBD_DP_PULL_UP_PORT->BRR = USBD_DP_PULL_UP_PIN;
#else
	USBD_DP_PULL_UP_PORT->BSRR = USBD_DP_PULL_UP_PIN;
#endif
}
#endif


/****************************************************************************/
/*               USBD_VBUS_DETECT                                           */
/****************************************************************************/

#if(USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_ON)
/** 
 *  Sprawdzenie, czy na linii VBUS wystepuje napiecie zasilania od Hosta
*/
uint8_t USBD_device_is_powered(void) {
	if((USBD_VBUS_PORT->IDR & USBD_VBUS_PIN)) {
#if(USBD_VBUS_ACTIVE_STATE)
		return(1);
#else
		return(0);
#endif
	}
	else {
#if(USBD_VBUS_ACTIVE_STATE)
		return(0);
#else
		return(1);
#endif
	}
}

/** 
 *  Przerwanie EXTI od wykrycia napiecia na linii VBUS
*/
void EXTI4_IRQHandler(void) {
	// skasowanie flagi przerwania
	EXTI_ClearITPendingBit(USBD_VBUS_EXTI_Line);

	USBD_Attach_irq_handler();
}

/** 
 *  Konfiguruje pin i przerwanie VBUS DETECT
*/
static void USBD_VBUS_DETECT_configure_pin_and_irq(void) {
	GPIO_InitTypeDef gpio_data;
	NVIC_InitTypeDef USBD_NVIC_InitStructure;

	// wybor portu z ktorego podpinamy przerwanie
	GPIO_EXTILineConfig(USBD_VBUS_GPIO_PortSource, USBD_VBUS_GPIO_PinSource);

	// inicjalizacja portu linii reagujacej
	// przerwaniem na pojawienie sie / znikniecie
	// zasilania na linii USB
	gpio_data.GPIO_Pin = USBD_VBUS_PIN;
	gpio_data.GPIO_Speed = GPIO_Speed_50MHz;
	gpio_data.GPIO_Mode = USBD_VBUS_MODE;
	GPIO_Init(USBD_VBUS_PORT, &gpio_data);

	// konfiguracja przerwania VBUS w kontrolerze
	// przerwan NVIC
	USBD_NVIC_InitStructure.NVIC_IRQChannel = USBD_VBUS_EXTI_IRQChannel;
	USBD_NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	USBD_NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	USBD_NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&USBD_NVIC_InitStructure);
}

/** 
 *  Przekonfigurowuje pin i przerwanie VBUS DETECT
 * 
 *  \param state informuje, czy przerwanie ma byc wlaczone czy wylaczone
*/
void USBD_VBUS_DETECT_reconfigure_irq(uint8_t state) {
	EXTI_InitTypeDef USBD_EXTInit;

	//wlaczenie przerwania wykwywajacego stan linii VBUS
	USBD_EXTInit.EXTI_Line = USBD_VBUS_EXTI_Line;
	if(state == USBD_FEATURE_ON) {
		USBD_EXTInit.EXTI_LineCmd = ENABLE;
	}
	else {
		USBD_EXTInit.EXTI_LineCmd = DISABLE;
	}
	USBD_EXTInit.EXTI_Mode = EXTI_Mode_Interrupt;
	USBD_EXTInit.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
	EXTI_Init(&USBD_EXTInit);
}
#endif


/****************************************************************************/
/*               USBD peripheral manage                                     */
/****************************************************************************/

/** 
 *  Inicjalizuje peryferium USB oraz wszystkie potrzebne elementy
*/
void USBD_low_level_init(void) {
	NVIC_InitTypeDef USBD_NVIC_InitStructure;
#if (((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) || (USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON)) &&  (USBD_MAX_NUM_EVENTS > 0))
	EXTI_InitTypeDef USBD_EXTI_InitStructure;
#endif

#if(USBD_DP_PULL_UP_SUPPORT == USBD_FEATURE_ON)
	// inicjalizacja portu linii sterujacej
	// rezystorem Pull-Up na linii D+
	USBD_DP_PULL_UP_configure_pin();
#endif
	
	// USBCLK = PLLCLK / USBD_RCC_USBCLKSource_PLLCLK_presc
	*(volatile uint32_t *) CFGR_USBPRE_BB = USBD_RCC_USBCLKSource_PLLCLK_presc;
//	RCC_USBCLKConfig(USBD_RCC_USBCLKSource_PLLCLK_presc);

	// konfiguracja przerwania USB WakeUp
	USBD_NVIC_InitStructure.NVIC_IRQChannel = USBWakeUp_IRQn;
	USBD_NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	USBD_NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	USBD_NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&USBD_NVIC_InitStructure);

#if (((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) || (USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON)) &&  (USBD_MAX_NUM_EVENTS > 0))
	USBD_EXTI_InitStructure.EXTI_Line = EXTI_Line18;
	USBD_EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
	USBD_EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
	USBD_EXTI_InitStructure.EXTI_LineCmd = ENABLE;
	EXTI_Init(&USBD_EXTI_InitStructure);
#endif

	// konfiguracja przerwania USB nizszego priorytetu
	USBD_NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
	USBD_NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
	USBD_NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	USBD_NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&USBD_NVIC_InitStructure);

	// konfiguracja przerwania USB wyzszego priorytetu
	USBD_NVIC_InitStructure.NVIC_IRQChannel = USB_HP_CAN1_TX_IRQn;
	USBD_NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	USBD_NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	USBD_NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&USBD_NVIC_InitStructure);


#if(USBD_VBUS_DETECT_SUPPORT == USBD_FEATURE_ON)
	// inicjalizacja portu linii reagujacej
	// przerwaniem na pojawienie sie / znikniecie
	// zasilania na linii USB oraz konfiguracja
	// przerwania VBUS w kontrolerze przerwan NVIC
	USBD_VBUS_DETECT_configure_pin_and_irq();
#endif
}


#if (((USBDE_WAKEUP_SUPPORT == USBD_FEATURE_ON) || (USBDE_SUSPENDED_SUPPORT == USBD_FEATURE_ON)) &&  (USBD_MAX_NUM_EVENTS > 0))
/** 
 *  Czyści flagi przerwania EXTI Wakeup
*/
void USBD_clear_wakeup_exti_status(void) {
	if(EXTI_GetITStatus(EXTI_Line18) != RESET) {
		EXTI_ClearITPendingBit(EXTI_Line18);
	}
}
#endif


/** 
 *  Wybudza peryfyferium USB z trybu uśpienia
*/
void USBD_power_up(void) {
	// wlaczenie zegara USBD
	RCC->APB1ENR |= RCC_APB1Periph_USB;
//	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, ENABLE);

	// zresetowanie USBD
	RCC->APB1RSTR |= RCC_APB1Periph_USB;
//	RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, ENABLE);

	// wylaczenie resetu
	RCC->APB1RSTR &= ~RCC_APB1Periph_USB;
//	RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, DISABLE);

	// wylaczenie trybu power down
	USBD_STM32_REG->CNTR &= ~((uint32_t)USBD_CNTR_PDWN);
	while(USBD_STM32_REG->CNTR & USBD_CNTR_PDWN) {
		;
	}

	// wylaczenie resetu peryferium
	USBD_STM32_REG->CNTR &= ~((uint32_t)USBD_CNTR_FRES);

	// skasowanie wszystkich flag przerwan
	USBD_STM32_REG->ISTR = 0;

	// zezwolenie na przerwania od resetu z linii USB
	USBD_STM32_REG->CNTR |= USBD_CNTR_RESETM;
}

/** 
 *  Wprowadza peryfyferium USB w tryb uśpienia
*/
void USBD_power_down(void) {
	// zresetowanie USBD
	RCC->APB1RSTR |= RCC_APB1Periph_USB;
//	RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, ENABLE);

	// wylaczenie resetu
	RCC->APB1RSTR &= ~RCC_APB1Periph_USB;
//	RCC_APB1PeriphResetCmd(RCC_APB1Periph_USB, DISABLE);

	// wylaczenie zegara USBD
	RCC->APB1ENR &= ~RCC_APB1Periph_USB;
//	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USB, DISABLE);
}
