/*
================================================================================
 File: usbd_stm32_hardware.c
--------------------------------------------------------------------------------
 Author:        Piotr Wojtowicz
 Purpose:       USBD STM32 hardware definitions
 Compiler:      ANSI C
 Hardware:      
 Doc:
--------------------------------------------------------------------------------
 Includes:      
--------------------------------------------------------------------------------
 Comments:                      UWAGA!!!
                Oprogramowanie to udostępnione jest do celów poglądowych,
                nie jest przetestowane pod kątem bezpieczeństwa ani spełnienia
                jakichkolwiek norm wymaganych aby urządzenie używające go
                mogło być dopuszczone do użytku, dlatego używasz go
                na własną odpowiedzialność a jego autor nie ponosi żadnego
                typu odpowiedzialności za jakiekolwiek skutki jego użycia.
                Jeżeli nie zgadzasz się z powyższym warunkiem proszę
                o nieużywanie tego oprogramowania.
================================================================================
*/

#ifndef USBD_STM32_HARDWARE_H_
#define USBD_STM32_HARDWARE_H_

#include <stdint.h>

/**
 * Registers bits definitions
 */
#define USBDREG_BIT_15          (1 << 15)
#define USBDREG_BIT_14          (1 << 14)
#define USBDREG_BIT_13          (1 << 13)
#define USBDREG_BIT_12          (1 << 12)
#define USBDREG_BIT_11          (1 << 11)
#define USBDREG_BIT_10          (1 << 10)
#define USBDREG_BIT_9           (1 <<  9)
#define USBDREG_BIT_8           (1 <<  8)
#define USBDREG_BIT_7           (1 <<  7)
#define USBDREG_BIT_6           (1 <<  6)
#define USBDREG_BIT_5           (1 <<  5)
#define USBDREG_BIT_4           (1 <<  4)
#define USBDREG_BIT_3           (1 <<  3)
#define USBDREG_BIT_2           (1 <<  2)
#define USBDREG_BIT_1           (1 <<  1)
#define USBDREG_BIT_0           (1 <<  0)

#define USBDREG_BITS_3_TO_0     (USBDREG_BIT_3 | USBDREG_BIT_2 | USBDREG_BIT_1 | USBDREG_BIT_0)
#define USBDREG_BITS_6_TO_0     (USBDREG_BIT_6 | USBDREG_BIT_5 | USBDREG_BIT_4 | USBDREG_BITS_3_TO_0)
#define USBDREG_BITS_10_TO_0    (USBDREG_BIT_10 | USBDREG_BIT_9 | USBDREG_BIT_8 | USBDREG_BIT_7 | USBDREG_BITS_6_TO_0)


/**
 * ISTR register bits definitions
 */
#define USBD_ISTR_CTR           (USBDREG_BIT_15)        // Correct TRansfer
#define USBD_ISTR_PMAOVR        (USBDREG_BIT_14)        // DMA Over/underrun
#define USBD_ISTR_ERR           (USBDREG_BIT_13)        // Error
#define USBD_ISTR_WKUP          (USBDREG_BIT_12)        // WaKe UP
#define USBD_ISTR_SUSP          (USBDREG_BIT_11)        // Suspend
#define USBD_ISTR_RESET         (USBDREG_BIT_10)        // Reset
#define USBD_ISTR_SOF           (USBDREG_BIT_9)         // Start Of Frame
#define USBD_ISTR_ESOF          (USBDREG_BIT_8)         // Expected Start Of Frame
#define USBD_ISTR_DIR           (USBDREG_BIT_4)         // Direction of transaction
#define USBD_ISTR_EP_ID         (USBDREG_BITS_3_TO_0)   // Endpoint Identifier


/**
 * CNTR register bits definitions
 */
#define USBD_CNTR_CTRM          (USBDREG_BIT_15)        // Correct Transfer Mask
#define USBD_CNTR_PMAOVRM       (USBDREG_BIT_14)        // DMA Over/underrun Mask
#define USBD_CNTR_ERRM          (USBDREG_BIT_13)        // Error Mask
#define USBD_CNTR_WKUPM         (USBDREG_BIT_12)        // WaKe UP Mask
#define USBD_CNTR_SUSPM         (USBDREG_BIT_11)        // Suspend Mask
#define USBD_CNTR_RESETM        (USBDREG_BIT_10)        // RESET Mask
#define USBD_CNTR_SOFM          (USBDREG_BIT_9)         // Start Of Frame Mask
#define USBD_CNTR_ESOFM         (USBDREG_BIT_8)         // Expected Start Of Frame Mask
                                                        // USBDREG_BIT_7 reserved
                                                        // USBDREG_BIT_6 reserved
                                                        // USBDREG_BIT_5 reserved
#define USBD_CNTR_RESUME        (USBDREG_BIT_4)         // RESUME request
#define USBD_CNTR_FSUSP         (USBDREG_BIT_3)         // Force Suspend
#define USBD_CNTR_LPMODE        (USBDREG_BIT_2)         // Low-power Mode
#define USBD_CNTR_PDWN          (USBDREG_BIT_1)         // Power Down
#define USBD_CNTR_FRES          (USBDREG_BIT_0)         // Force USB Reset


/**
 * FNR register bits definitions
 */
#define USBD_FNR_RXDP           (USBDREG_BIT_15)                    // status of D+ data line
#define USBD_FNR_RXDM           (USBDREG_BIT_14)                    // status of D- data line
#define USBD_FNR_LCK            (USBDREG_BIT_13)                    // Locked
#define USBD_FNR_LSOF           (USBDREG_BIT_12 | USBDREG_BIT_11)   // Lost SOF
#define USBD_FNR_FN             (USBDREG_BITS_10_TO_0)              // Frame Number


/**
 * DADDR register bits definitions
 */
#define USBD_DADDR_EF           (USBDREG_BIT_7)                     // Enable Function
#define USBD_DADDR_ADD          (USBDREG_BITS_6_TO_0)               // Device Address


/**
 * Endpoint register bits definitions
 */
#define USBDEP_STM32_CTR_RX     (USBDREG_BIT_15)                    // Endpoint Correct TRansfer RX
#define USBDEP_STM32_DTOG_RX    (USBDREG_BIT_14)                    // Endpoint Data Toggle RX
#define USBDEP_STM32_STAT_RX    (USBDREG_BIT_13 | USBDREG_BIT_12)   // Endpoint RX Status Field
#define USBDEP_STM32_SETUP      (USBDREG_BIT_11)                    // Endpoint Setup
#define USBDEP_STM32_TYPE       (USBDREG_BIT_10 | USBDREG_BIT_9)    // Endpoint Type
#define USBDEP_STM32_KIND       (USBDREG_BIT_8)                     // Endpoint Kind
#define USBDEP_STM32_CTR_TX     (USBDREG_BIT_7)                     // Endpoint Correct TRansfer TX
#define USBDEP_STM32_DTOG_TX    (USBDREG_BIT_6)                     // Endpoint Data Toggle TX
#define USBDEP_STM32_STAT_TX    (USBDREG_BIT_5  | USBDREG_BIT_4)    // Endpoint TX Status Field
#define USBDEP_STM32_ADDR       (USBDREG_BITS_3_TO_0)               // Endpoint Address Field


/**
 * Endpoint Type Field Definitions
 */
#define USBDEP_STM32_BULK        (0x0000)                           // Endpoint BULK
#define USBDEP_STM32_CONTROL     (USBDREG_BIT_9)                    // Endpoint CONTROL
#define USBDEP_STM32_ISOCHRONOUS (USBDREG_BIT_10)                   // Endpoint ISOCHRONOUS
#define USBDEP_STM32_INTERRUPT   (USBDREG_BIT_10 | USBDREG_BIT_9)   // Endpoint INTERRUPT


/**
 * Endpoint TX Status Field Definitions
 */
#define USBDEP_STM32_TX_DIS      (0x0000)                           // Endpoint Tx DisSabled
#define USBDEP_STM32_TX_STALL    (USBDREG_BIT_4)                    // Endpoint Tx Stalled
#define USBDEP_STM32_TX_NAK      (USBDREG_BIT_5)                    // Endpoint Tx Naked
#define USBDEP_STM32_TX_VALID    (USBDREG_BIT_5 | USBDREG_BIT_4)    // Endpoint Tx Valid


/**
 * Endpoint RX Status Field Definitions
 */
#define USBDEP_STM32_RX_DIS      (0x0000)                           // Endpoint RX DISabled
#define USBDEP_STM32_RX_STALL    (USBDREG_BIT_12)                   // Endpoint RX STALLed
#define USBDEP_STM32_RX_NAK      (USBDREG_BIT_13)                   // Endpoint RX NAKed
#define USBDEP_STM32_RX_VALID    (USBDREG_BIT_13 | USBDREG_BIT_12)  // Endpoint RX VALID


#define USBD_REG_BASE			0x40005C00
#define USBD_BUFFER_BASE		0x40006000


typedef struct {
	volatile uint32_t EP_REG[8];
	volatile uint32_t regs_reserved[8];
	volatile uint32_t CNTR;
	volatile uint32_t ISTR;
	volatile uint32_t FNR;
	volatile uint32_t DADDR;
	volatile uint32_t BTABLE;
}USBD_STM32_registers;


typedef union {
	struct {
		volatile uint32_t tx_addr;
		volatile uint32_t tx_count;
		volatile uint32_t rx_addr;
		volatile uint32_t rx_count;
	}SB;
	struct {
		volatile uint32_t addr;
		volatile uint32_t count;
	}DB[2];
}USBDEP_STM32_MEM_descriptor;


#define USBD_STM32_REG		((USBD_STM32_registers *) USBD_REG_BASE)

#define USBD_STM32_EPMEM	((USBDEP_STM32_MEM_descriptor *) USBD_BUFFER_BASE)







#define USBDEP_SB_set_tx_count(EP_number, Count)	USBD_STM32_EPMEM[EP_number].SB.tx_count = Count
#define USBDEP_DB_set_tx_count(EP_number, Count) 	USBD_STM32_EPMEM[EP_number].DB[(USBD_STM32_REG->EP_REG[EP_number] & USBDEP_STM32_DTOG_RX) >> 14].count = Count
#define USBDEP_ISO_set_tx_count(EP_number, Count) 	USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_TX) & USBDEP_STM32_DTOG_TX) >> 6].count = Count

#define USBDEP_DB_get_tx_count_HB(EP_number) 		((uint16_t)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_RX) & USBDEP_STM32_DTOG_RX) >> 14].count) & 0x3ff)
#define USBDEP_ISO_get_tx_count_HB(EP_number) 		((uint16_t)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number]) & USBDEP_STM32_DTOG_TX) >> 6].count) & 0x3ff)

#define USBDEP_SB_get_rx_count(EP_number)			((uint16_t)(USBD_STM32_EPMEM[EP_number].SB.rx_count) & 0x3ff)
#define USBDEP_DB_get_rx_count(EP_number) 			((uint16_t)(USBD_STM32_EPMEM[EP_number].DB[(USBD_STM32_REG->EP_REG[EP_number] & USBDEP_STM32_DTOG_TX) >> 6].count) & 0x3ff)
#define USBDEP_DB_get_rx_count_HB(EP_number) 		((uint16_t)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_TX) & USBDEP_STM32_DTOG_TX) >> 6].count) & 0x3ff)
#define USBDEP_ISO_get_rx_count(EP_number) 			((uint16_t)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_RX) & USBDEP_STM32_DTOG_RX) >> 14].count) & 0x3ff)





// UWAGA na to makro!!!
// modyfikune parametr Count, dlatego w celu podania go
// nalezy posluzyc sie stworzona do tego celu zmienna!!!
#define USBDEP_SB_set_rx_max_possible_count(EP_number, Count)	{\
	if(Count > 62) {\
		Count >>= 5;\
		Count--;\
		Count <<= 10;\
		Count |= (1 << 15);\
	}\
	else {\
		Count <<= 9;\
	}\
	USBD_STM32_EPMEM[EP_number].SB.rx_count = (Count & 0xFC00);\
}
// UWAGA na to makro!!!
// modyfikune parametr Count, dlatego w celu podania go
// nalezy posluzyc sie stworzona do tego celu zmienna!!!
#define USBDEP_DB_set_first_buf_max_possible_count(EP_number, Count)	{\
	if(Count > 62) {\
		Count >>= 5;\
		Count--;\
		Count <<= 10;\
		Count |= (1 << 15);\
	}\
	else {\
		Count <<= 9;\
	}\
	USBD_STM32_EPMEM[EP_number].SB.tx_count = (Count & 0xFC00);\
}




#define USBDEP_DB_get_tx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].DB[(USBD_STM32_REG->EP_REG[EP_number] & USBDEP_STM32_DTOG_RX) >> 14].addr * 2))
#define USBDEP_ISO_get_tx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_TX) & USBDEP_STM32_DTOG_TX) >> 6].addr * 2))
#define USBDEP_SB_get_tx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].SB.tx_addr * 2))

#define USBDEP_DB_get_rx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].DB[(USBD_STM32_REG->EP_REG[EP_number] & USBDEP_STM32_DTOG_TX) >> 6].addr * 2))
#define USBDEP_DB_get_rx_buffer_addr_HB(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_TX) & USBDEP_STM32_DTOG_TX) >> 6].addr * 2))
#define USBDEP_ISO_get_rx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].DB[((USBD_STM32_REG->EP_REG[EP_number] ^ USBDEP_STM32_DTOG_RX) & USBDEP_STM32_DTOG_RX) >> 14].addr * 2))
#define USBDEP_SB_get_rx_buffer_addr(EP_number) 	(USBD_BUFFER_BASE + (uint8_t *)(USBD_STM32_EPMEM[EP_number].SB.rx_addr * 2))



#define USBDEP_set_rx_buf_addr(EP_number, Addr)		USBD_STM32_EPMEM[EP_number].SB.rx_addr = (Addr & 0xFFFE)
#define USBDEP_set_tx_buf_addr(EP_number, Addr)		USBD_STM32_EPMEM[EP_number].SB.tx_addr = (Addr & 0xFFFE)


#define USBDEP_reg_modify(EP_number, mirror, EX_OR_mask, AND_mask, OR_mask) \
	(mirror) = USBD_STM32_REG->EP_REG[EP_number];\
	(mirror) ^= (EX_OR_mask);\
	(mirror) &= (AND_mask);\
	(mirror) |= (OR_mask);\
	USBD_STM32_REG->EP_REG[EP_number] = (mirror)
	





#endif /*USBD_STM32_HARDWARE_H_*/
